unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ComCtrls, SyncObjs;

const WM_UPDATE_PROGRESS = WM_APP + 100;

type
  TDemoThread = class( TThread )
  protected
    procedure Execute(); override;

  end;

  TForm1 = class(TForm)
    StartButton: TButton;
    ProgressBar1: TProgressBar;
    StopButton: TButton;
    procedure StartButtonClick(Sender: TObject);
    procedure StopButtonClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    FThread   : TDemoThread;
    FProgress : Integer;
    FSection  : TCriticalSection;

  private
    procedure WMUpdateProgress(var Message: TMessage); message WM_UPDATE_PROGRESS;
    
  private
    procedure DoTerminate(Sender: TObject);

  public
    procedure UpdateProgress( AProgress : Integer );

  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

procedure TForm1.StartButtonClick(Sender: TObject);
begin
  StartButton.Enabled := False;
  StopButton.Enabled := True;

  FThread := TDemoThread.Create( True );
  FThread.FreeOnTerminate := True;
  FThread.OnTerminate := DoTerminate;
  FThread.Resume();

end;

procedure TForm1.DoTerminate(Sender: TObject);
begin
  FThread := NIL;
  StopButton.Enabled := False;
  StartButton.Enabled := True;
end;

procedure TForm1.StopButtonClick(Sender: TObject);
begin
  if( FThread <> NIL ) then
    FThread.Terminate();

end;

procedure TForm1.UpdateProgress( AProgress : Integer );
begin
  FSection.Enter();
  FProgress := AProgress;
  FSection.Leave();
  PostMessage( Handle, WM_UPDATE_PROGRESS, 0, 0 );

//  PostMessage( Handle, WM_UPDATE_PROGRESS, AProgress, 0 );
end;

procedure TForm1.WMUpdateProgress(var Message: TMessage);
var
  AProgress : Integer;
  
begin
  FSection.Enter();
  AProgress := FProgress;
  FSection.Leave();

  ProgressBar1.Position := AProgress;

//  ProgressBar1.Position := Message.WParam;
end;

procedure TDemoThread.Execute();
var
  I : Integer;
  J : Integer;
  F : TextFile;
  S : String;


begin
  for J := 0 to 100 do
    begin
    if( Terminated ) then
      Break;

    AssignFile( F, 'Data.txt' );
    Reset( F );

    for I := 0 to 1000000 do
      Readln(F, S );

    CloseFile(F);
    Form1.UpdateProgress( J );
    end;

end;


procedure TForm1.FormCreate(Sender: TObject);
begin
  FSection := TCriticalSection.Create();
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  FSection.Free();
end;

end.
