unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, SyncObjs, StdCtrls;

const
  THREAD_COUNT = 10;

type
  TForm1 = class;

  TThreadMessage = record
    FChannelNo     : Integer;
    FMessageBuffer : array of Byte;

  end;

  TMessageQueue = record
    FLock     : TCriticalSection;
    FMessages : array of TThreadMessage;
  end;

  TComThread = class( TThread )
  private
    FOwner        : TForm1;
    FThreadNo     : Integer;
    FMessageQueue : TMessageQueue;

  protected
    procedure Execute(); override;

  public
    constructor Create( AOwner : TForm1; AThreadNo : Integer );
    destructor  Destroy(); override;

  end;

  TForm1 = class(TForm)
    ListBox1: TListBox;
    Label1: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    FThreads : array [ 0..THREAD_COUNT-1 ] of TComThread;

  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

constructor TComThread.Create( AOwner : TForm1; AThreadNo : Integer );
begin
  inherited Create( True );
  FOwner := AOwner;
  FThreadNo := AThreadNo;
  FMessageQueue.FLock := TCriticalSection.Create();
  end;

destructor TComThread.Destroy();
begin
  FMessageQueue.FLock.Free();
  inherited;
end;

procedure TComThread.Execute();
var
  I             : Integer;
  AReceiver     : TComThread;
  ANextReceiver : Integer;
  ACounter      : Integer;

begin
  while( not Terminated ) do
    begin
    FMessageQueue.FLock.Enter();
    if( Random( 10 ) = 1 ) then
      begin
      SetLength( FMessageQueue.FMessages, Length( FMessageQueue.FMessages ) + 1 );
      while( True ) do
        begin
        ANextReceiver := Random( THREAD_COUNT );
        if( ANextReceiver <> FThreadNo ) then
          begin
          FMessageQueue.FMessages[ Length( FMessageQueue.FMessages ) - 1 ].FChannelNo := ANextReceiver;
          Break;
          end;
        end;
      end;

    for I := 0 to Length( FMessageQueue.FMessages ) - 1 do
      begin
      AReceiver := FOwner.FThreads[ FMessageQueue.FMessages[ I ].FChannelNo ];
      AReceiver.FMessageQueue.FLock.Enter();
      SetLength( AReceiver.FMessageQueue.FMessages, Length( AReceiver.FMessageQueue.FMessages ) + 1 );
      SetLength( AReceiver.FMessageQueue.FMessages[ Length( AReceiver.FMessageQueue.FMessages ) - 1 ].FMessageBuffer, Length( FMessageQueue.FMessages[ I ].FMessageBuffer ));
      Move( FMessageQueue.FMessages[ I ].FMessageBuffer[ 0 ], AReceiver.FMessageQueue.FMessages[ Length( AReceiver.FMessageQueue.FMessages ) - 1 ].FMessageBuffer[ 0 ], Length( FMessageQueue.FMessages[ I ].FMessageBuffer ));

      while( True ) do
        begin
        ANextReceiver := Random( THREAD_COUNT );
        if( ANextReceiver <> FThreadNo ) then
          begin
          AReceiver.FMessageQueue.FMessages[ Length( AReceiver.FMessageQueue.FMessages ) - 1 ].FChannelNo := ANextReceiver;
          Break;
          end;
        end;

      AReceiver.FMessageQueue.FLock.Leave();
      end;

    SetLength( FMessageQueue.FMessages, 0 );
    FMessageQueue.FLock.Leave();
    Synchronize(
      procedure()
      begin
        FOwner.ListBox1.Items[ FThreadNo ] := IntToStr( ACounter );
      end
      );

    Inc( ACounter );
    Sleep( Random( 30 ) );
    end;

end;

procedure TForm1.FormCreate(Sender: TObject);
var
  I : Integer;

begin
  for I := 0 to THREAD_COUNT - 1 do
    begin
    ListBox1.Items.Add( '0' );
    FThreads[ I ] := TComThread.Create( Self, I );
    end;

  for I := 0 to THREAD_COUNT - 1 do
    FThreads[ I ].Resume();

end;

procedure TForm1.FormDestroy(Sender: TObject);
var
  I : Integer;

begin
  for I := 0 to THREAD_COUNT - 1 do
    begin
    FThreads[ I ].Terminate();
    FThreads[ I ].WaitFor();
    end;

  for I := 0 to THREAD_COUNT - 1 do
    FThreads[ I ].Free();

end;

end.
