unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, SyncObjs, StdCtrls, ExtCtrls;

const
  COUNT_READERS = 10;
  READER_DATA_SIZE = 100;

type
  TForm1 = class;
  
  TSharedData = record
    FLock : TMultiReadExclusiveWriteSynchronizer;
    FData : array[ 0..9999 ] of Real;
  end;

  TReaderData = record
    FDataSection : TCriticalSection;
    FData : array[ 0..READER_DATA_SIZE - 1 ] of Real;
  end;

  TReaderThread = class( TThread )
  protected
    FOwner : TForm1;
    FData  : TReaderData;

  protected
    procedure Execute(); override;

  public
    constructor Create( AOwner : TForm1 );
    destructor  Destroy(); override;

  end;

  TWriterThread = class( TThread )
  protected
    FOwner      : TForm1;
    FStartEvent : TEvent;

  protected
    procedure Execute(); override;

  public
    procedure Terminate();
    procedure WakeUp();

  public
    constructor Create( AOwner : TForm1 );
    destructor  Destroy(); override;
    
  end;
  
  TForm1 = class(TForm)
    Button1: TButton;
    Timer1: TTimer;
    ListBox1: TListBox;
    Label1: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
  private
    FData : TSharedData;
    FReaders : array [0..COUNT_READERS - 1] of TReaderThread;
    FWriter  : TWriterThread;
    FDisplayIndex : Integer; 

  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

uses Math;

{$R *.dfm}

procedure TForm1.FormCreate(Sender: TObject);
var
  I : Integer;
  
begin
  FData.FLock := TMultiReadExclusiveWriteSynchronizer.Create();
  for I := 0 to COUNT_READERS - 1 do
    FReaders[ I ] := TReaderThread.Create( Self );

  FWriter := TWriterThread.Create( Self );
end;

procedure TForm1.FormDestroy(Sender: TObject);
var
  I : Integer;
  
begin
  FWriter.Terminate();
  FWriter.WaitFor();
  FWriter.Free();

  for I := 0 to COUNT_READERS - 1 do
    begin
    FReaders[ I ].Terminate();
    FReaders[ I ].WaitFor();
    FReaders[ I ].Free();
    end;
    
  FData.FLock.Free();
end;

constructor TReaderThread.Create( AOwner : TForm1 );
begin
  inherited Create( True );
  FOwner := AOwner;
  FData.FDataSection := TCriticalSection.Create();
  Resume();  
end;

destructor TReaderThread.Destroy();
begin
  FData.FDataSection.Free();
  inherited;
end;

procedure TReaderThread.Execute();
var
  I : Integer;
  
begin
  while( not Terminated ) do
    begin
    Sleep( 10 );
    FOwner.FData.FLock.BeginRead();
    FData.FDataSection.Enter();
    for I := 0 to Length( FData.FData ) - 1 do
      FData.FData[ I ] := FOwner.FData.FData[ RandomRange( 0, Length( FOwner.FData.FData ) ) ];
       
    FData.FDataSection.Leave();
    FOwner.FData.FLock.EndRead();
    end;
    
end;

constructor TWriterThread.Create( AOwner : TForm1 );
begin
  inherited Create( True );
  FOwner := AOwner;
  FStartEvent := TEvent.Create( NIL, False, False, '' );
  Resume();  
end;

destructor TWriterThread.Destroy();
begin
  FStartEvent.Free();
  inherited;
end;

procedure TWriterThread.Terminate();
begin
  inherited Terminate();
  FStartEvent.SetEvent();
end;

procedure TWriterThread.WakeUp();
begin
  FStartEvent.SetEvent();
end;

procedure TWriterThread.Execute();
var
  I : Integer;
  
begin
  while( not Terminated ) do
    begin
    FStartEvent.WaitFor( INFINITE );
        
    FOwner.FData.FLock.BeginWrite();

    for I := 0 to Length( FOwner.FData.FData ) - 1 do
      FOwner.FData.FData[ I ] := RandomRange( -1000, 1000 );

    FOwner.FData.FLock.EndWrite();
    end;

end;

procedure TForm1.Button1Click(Sender: TObject);
begin
  FWriter.WakeUp();
end;

procedure TForm1.Timer1Timer(Sender: TObject);
var
  AData : array[ 0..READER_DATA_SIZE - 1 ] of Real;
  I : Integer;

begin
  FReaders[ FDisplayIndex ].FData.FDataSection.Enter();
  Move( FReaders[ FDisplayIndex ].FData.FData[ 0 ], AData[ 0 ], SizeOf( AData )); 
  FReaders[ FDisplayIndex ].FData.FDataSection.Leave();

  Label1.Caption := 'Data from thread #' + IntToStr( FDisplayIndex );
  ListBox1.Items.BeginUpdate();
  ListBox1.Items.Clear(); 
  for I := 0 to READER_DATA_SIZE - 1 do
    ListBox1.Items.Add( FloatToStr( AData[ I ] ));
  
  ListBox1.Items.EndUpdate();

  Inc( FDisplayIndex );
  if( FDisplayIndex >= Length( FReaders ) ) then
    FDisplayIndex := 0;
      
end;

end.
 